#!/usr/bin/env python
# -*- coding: cp1252 -*-

import pyglet
from pyglet.window import key
from pyglet.window import mouse

import imageLoader
import callback

def callbackSpriteA(sprite):
    print 'callback Sprite A: '
    print sprite.id

class MainWindow(pyglet.window.Window):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.sprites = []
        self.backgroundSprites = {}
        self.foregroundSprites = []
        self.mobSprites = []

        self.background = pyglet.graphics.OrderedGroup(0)
        self.foreground = pyglet.graphics.OrderedGroup(1)
        self.mob = pyglet.graphics.OrderedGroup(2)

        self.sizeCase = 50
        self.delta = 0.75

        self.numberHCase = self.height / 50 + 1
        self.numberWCase = self.width / 50 + 1

        self.offsetH = 0
        self.offsetW = 0
        
        self.loader = imageLoader.ImageLoader()

        pyglet.clock.schedule_interval(self.updateGraphics, 0.001)
        
    def on_draw(self):
        self.clear()
        for sprit in self.sprites:
            sprit.draw()

    def on_mouse_press(self, x, y, button, modifiers):
        if button == mouse.LEFT:
            for sprite in self.sprites:
                if(sprite.x <= x and
                   sprite.x + sprite.width >= x and
                   sprite.y <= y and
                   sprite.y + sprite.height >= y):
                    sprite.callback(sprite)

                
    def addImage(self, fileName, x, y, level):
        if x > self.numberWCase:
            return
        if y > self.numberHCase:
            return
        
        sprite = self.loader.sprite(fileName)
        sprite.x = x * 50
        sprite.y = y * 50
        sprite.fileName = fileName

        if (level == 'background'):
            sprite.group = self.background
            self.backgroundSprites[(x, y)] = sprite
        if (level == 'foreground'):
            sprite.group = self.foreground
            self.foregroundSprites.append(sprite)
        if (level == 'mob'):
            sprite.group = self.mob
            self.mobSprites.append(sprite)
        
        self.sprites.append(sprite)
        return sprite

    def register(self, imageList):
        for (name, callback, size) in imageList:
            self.loader.register(name, callback, size)

    def dropAllSprits(self):
        self.sprites = []
        self.backgroundSprites = {}
        self.foregroundSprites = []
        self.mobSprites = []
        self.on_draw()

    def buildBackground(self, spritList):
        for (x, y), name in spritList.items():
            self.addImage(name, x, y, 'background')
    
    def updateGraphics(self, dt):
        for sprit in self.mobSprites:
            if hasattr(sprit, 'speedX'):
                if sprit.speedX != 0 :
                    sprit.x += dt * sprit.speedX
                    sprit.draw()
                    if (sprit.x > self.width - self.sizeCase
                    or sprit.x < self.sizeCase) :
                        self.deleteSprite(sprit)
                        continue
            if hasattr(sprit, 'speedY'):
                if sprit.speedY != 0 :
                    sprit.y += dt * sprit.speedY
                    sprit.draw()
                    if (sprit.y > self.height - self.sizeCase
                    or sprit.y < self.sizeCase) :
                        self.deleteSprite(sprit)
                        continue
            self.collision(sprit)

    def deleteSprite(self, sprit):
        self.deleteSpriteInList(sprit, self.sprites)
        self.deleteSpriteInList(sprit, self.mobSprites)
        self.deleteSpriteInList(sprit, self.foregroundSprites)
        self.deleteSpriteInList(sprit, self.backgroundSprites)

    def deleteSpriteInList(self, sprit, spritList):
        for internalSprite in spritList:
            if id(internalSprite) == id(sprit):
                spritList.remove(internalSprite)
                return True
        return False

    def collision(self, sprit):
        x = round(sprit.x / self.sizeCase)
        y = round(sprit.y / self.sizeCase)
        
        if hasattr(sprit, 'speedX') :
            if (sprit.x % self.sizeCase) <= self.delta and sprit.speedX > 0:
                self.checkCollision(sprit, x+1, y)
                
            if (sprit.x % self.sizeCase) >= self.sizeCase - self.delta and sprit.speedX < 0:
                self.checkCollision(sprit, x-1, y)
                
        if hasattr(sprit, 'speedY') :
            if sprit.y % self.sizeCase <= self.delta and sprit.speedY > 0:
                self.checkCollision(sprit, x, y+1)
                
            if sprit.y % self.sizeCase >= self.sizeCase - self.delta and sprit.speedY < 0:
                self.checkCollision(sprit, x, y-1)
                
    def checkCollision(self, mob, x, y):
        print ('checkCollision', x, y)
        for sprite in self.sprites:
            if ((sprite.x <= x * self.sizeCase + self.delta and sprite.x >= x * self.sizeCase - self.delta)
            and (sprite.y <= y * self.sizeCase + self.delta and sprite.y >= y * self.sizeCase - self.delta)):
                callback.collision(mob, sprite)


                
